#!/usr/bin/env python

# author: Pocholo
# date: 11/16/20

# Inspire by Conbagui tutorial found here: https://www.deviantart.com/conbagui/art/Gimp-2-10-tutorial-How-to-create-an-Orange-767069232

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

from gimpfu import *

def pm_create_an_orange(tingeColor):

	#Variables
	width = 1920
	height = 1080
	
	#Set gimp to default
	pdb.gimp_context_set_defaults()
	pdb.gimp_context_set_default_colors()
	
	#Create the gradient background
	img = pdb.gimp_image_new(width, height, RGB)
	backLayer = pdb.gimp_layer_new(img, width, height, RGB_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
	pdb.gimp_layer_add_alpha(backLayer)
	pdb.gimp_image_add_layer(img, backLayer, 0)
	pdb.gimp_drawable_fill(backLayer, FILL_TRANSPARENT)
	pdb.gimp_context_set_foreground((220, 122, 27))
	pdb.gimp_context_set_background((235, 212, 150))
	pdb.gimp_edit_blend(backLayer, 0, LAYER_MODE_NORMAL, 
	                                                  0, 
													100, 
										              0, 
													  0, 
												  FALSE, 
												  FALSE, 
													  3,
												   0.20, 
												   TRUE, 
												    960, 
												      0, 
												    960, 
												   1075)	
													
	#Show process
	pdb.gimp_display_new(img)
	
	#Create the orange
	orangeLayer = pdb.gimp_layer_new(img, width, height, RGB_IMAGE, "Orange", 100, LAYER_MODE_NORMAL)
	pdb.gimp_layer_add_alpha(orangeLayer)
	pdb.gimp_image_add_layer(img, orangeLayer, 0)
	pdb.gimp_drawable_fill(orangeLayer, FILL_TRANSPARENT)
	pdb.script_fu_oval_path(img, orangeLayer, 
	                                'Orange',   #name
										 960,   #centerx 
										 540,   #centery 
										 350,   #top radius 
										 400,   #left radius 
										 400,   #bottom radius 
										 400,   #right radius 
										   0,   #rotation
									   FALSE,   #stroke path
								   (0, 0, 0),   #stroke color 
									    TRUE,   #fill path 
							  (255, 139, 4)),   #fill color
	
	#Duplicate the orange
	non_empty = pdb.gimp_edit_copy(orangeLayer)
	copylayer1 = pdb.gimp_layer_new(img, width, height, RGB_IMAGE, "Orange copy", 85, LAYER_MODE_SOFTLIGHT)
	pdb.gimp_image_add_layer(img, copylayer1, 0)
	pdb.gimp_layer_add_alpha(copylayer1)
	pdb.gimp_drawable_fill(copylayer1, FILL_TRANSPARENT)
	floating_sel = pdb.gimp_edit_paste(copylayer1, FALSE)
	pdb.gimp_floating_sel_anchor(floating_sel)
	
	#Create the orange's peel
	pdb.plug_in_hsv_noise(img, copylayer1, 2, 3, 4, 255)
	pdb.plug_in_gmic_qt(img, copylayer1, 1, 0, 
		"-v - -fx_light_relief 0.5,2,0.05,0,0,54,45,1,0,0,0.15")
	orangeLayer = pdb.gimp_image_merge_down(img, copylayer1, CLIP_TO_IMAGE)
	
	#Duplicate the orange
	non_empty = pdb.gimp_edit_copy(orangeLayer)
	copylayer2 = pdb.gimp_layer_new(img, width, height, RGB_IMAGE, "Orange copy", 30, LAYER_MODE_MULTIPLY)
	pdb.gimp_image_add_layer(img, copylayer2, 0)
	pdb.gimp_layer_add_alpha(copylayer2)
	pdb.gimp_drawable_fill(copylayer2, FILL_TRANSPARENT)
	floating_sel = pdb.gimp_edit_paste(copylayer2, FALSE)
	pdb.gimp_floating_sel_anchor(floating_sel)
	orangeLayer = pdb.gimp_image_merge_down(img, copylayer2, CLIP_TO_IMAGE)
		
	#Orange's freckles
	non_empty = pdb.gimp_edit_copy(orangeLayer)
	frecklesLayer = pdb.gimp_layer_new(img, width, height, RGB_IMAGE, "Orange's freckles", 100, LAYER_MODE_MULTIPLY)
	pdb.gimp_image_add_layer(img, frecklesLayer, 0)
	pdb.gimp_layer_add_alpha(frecklesLayer)
	pdb.gimp_drawable_fill(frecklesLayer, FILL_TRANSPARENT)
	floating_sel = pdb.gimp_edit_paste(frecklesLayer, FALSE)
	pdb.gimp_floating_sel_anchor(floating_sel)
	pdb.plug_in_hsv_noise(img, frecklesLayer, 1, 3, 4, 255)
	pdb.gimp_levels(frecklesLayer, HISTOGRAM_VALUE, 0, 10, 1.35, 0, 255)
	pdb.plug_in_gauss(img, frecklesLayer, 10, 10, 1)
	
	#Orange stains
	if tingeColor == True:
		pdb.gimp_image_select_item(img, 2, orangeLayer)
		tingeColor = pdb.gimp_layer_new(img, width, height, RGB_IMAGE, "Orange's Tinge color", 48, LAYER_MODE_HARDLIGHT)
		pdb.gimp_image_add_layer(img, tingeColor, 0)
		pdb.gimp_layer_add_alpha(tingeColor)
		pdb.gimp_drawable_fill(tingeColor, FILL_TRANSPARENT)
		pdb.gimp_context_set_foreground((255, 255, 255))
		pdb.gimp_edit_bucket_fill(tingeColor, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
		pdb.plug_in_plasma(img, tingeColor, 
										42,  #random seed
									   0.5)  #Turbulence
		pdb.gimp_selection_none(img)		

	#Inner Shadow
	innerLayer = pdb.gimp_layer_new(img, width, height, RGB_IMAGE, "Inner shadow", 60, LAYER_MODE_NORMAL)
	pdb.gimp_layer_add_alpha(innerLayer)
	pdb.gimp_image_add_layer(img, innerLayer, 0)
	pdb.gimp_drawable_fill(innerLayer, FILL_TRANSPARENT)
	pdb.gimp_image_select_item(img, 2, orangeLayer)
	pdb.gimp_context_set_foreground(("Black"))
	pdb.gimp_edit_bucket_fill(innerLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
	pdb.gimp_brushes_set_brush("2. Hardness 100")
	pdb.gimp_context_set_brush_size(800)
	pdb.gimp_context_set_brush_default_hardness()
	pdb.gimp_context_set_brush_spacing(0.01)
	pdb.gimp_context_set_brush_force(0.50)
	pdb.gimp_eraser_default(innerLayer, 2, [1000, 550])
	pdb.plug_in_gauss(img, innerLayer, 75, 75, 1)
	
	#Inner Glow								   
	innergLayer = pdb.gimp_layer_new(img, width, height, RGB_IMAGE, "Inner glow", 100, LAYER_MODE_SOFTLIGHT)
	pdb.gimp_layer_add_alpha(innergLayer)
	pdb.gimp_image_add_layer(img, innergLayer, 0)
	pdb.gimp_drawable_fill(innergLayer, FILL_TRANSPARENT)
	pdb.gimp_context_set_foreground(("Black"))
	pdb.gimp_edit_bucket_fill(innergLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)	
	pdb.gimp_selection_shrink(img, 10)
	pdb.gimp_edit_clear(innergLayer)
	pdb.gimp_selection_none(img)
	pdb.plug_in_gauss(img, innergLayer, 60, 60, 1)
			
	#Reflection
	reflectionLayer = pdb.gimp_layer_new(img, width, height, RGB_IMAGE, "Reflection", 45, LAYER_MODE_SOFTLIGHT)
	pdb.gimp_image_add_layer(img, reflectionLayer, 0)
	pdb.gimp_layer_add_alpha(reflectionLayer)
	pdb.gimp_drawable_fill(reflectionLayer, FILL_TRANSPARENT)
	pdb.gimp_brushes_set_brush("2. Hardness 050")
	pdb.gimp_context_set_brush_size(400)
	pdb.gimp_context_set_brush_default_hardness()
	pdb.gimp_context_set_brush_spacing(0.01)
	pdb.gimp_context_set_brush_force(0.50)
	pdb.gimp_context_set_foreground("White")
	pdb.gimp_paintbrush_default(reflectionLayer, 2, [1100, 500])
		
	#Make a new from visible layer
	pdb.gimp_item_set_visible(backLayer, FALSE)
	visiblelayer = pdb.gimp_layer_new_from_visible(img, img, "Orange complete")
	pdb.gimp_image_add_layer(img, visiblelayer, 0)
	pdb.gimp_layer_set_mode(visiblelayer, LAYER_MODE_NORMAL)
	pdb.gimp_item_set_visible(backLayer, TRUE)
	
	#Make all the rest invisible
	pdb.gimp_item_set_visible(orangeLayer, FALSE)
	pdb.gimp_item_set_visible(innerLayer, FALSE)
	pdb.gimp_item_set_visible(innergLayer, FALSE)
	pdb.gimp_item_set_visible(reflectionLayer, FALSE)
	pdb.gimp_item_set_visible(tingeColor, FALSE)
	pdb.gimp_item_set_visible(frecklesLayer, FALSE)
		
	if tingeColor == True:
		pdb.gimp_item_set_visible(tingeColor, FALSE)
		
	#Orange shadow
	pdb.script_fu_perspective_shadow(img, visiblelayer, 
	                                               110,  #angle
												   5.0,  #relative dist from horizon
												   0.7,  #relative length fron shadow
												    35,  #blur radius
											   "black",  #color
												    80,  #opacity
												     1,  #interpolation
											     FALSE)  #allow resizing
		
	#Orange top shoulder or grooves
	shoulderLayer = pdb.gimp_layer_new(img, width, height, RGB_IMAGE, "Fruit shoulder", 40, LAYER_MODE_SOFTLIGHT)
	pdb.gimp_image_add_layer(img, shoulderLayer, 0)
	pdb.gimp_layer_add_alpha(shoulderLayer)
	pdb.gimp_drawable_fill(shoulderLayer, FILL_TRANSPARENT)
	pdb.gimp_brushes_set_brush("Fruit shoulders")
	pdb.gimp_context_set_brush_size(400)
	pdb.gimp_context_set_brush_default_hardness()
	pdb.gimp_context_set_brush_spacing(0.01)
	pdb.gimp_context_set_brush_force(0.50)
	pdb.gimp_paintbrush_default(shoulderLayer, 2, [960, 225])
	pdb.plug_in_emboss(img, shoulderLayer, 
	                                    30,   #azimuth 
									   120,   #elevation 
										50,   #depth 
										 0)   #emboss

	#Drop shadow
	pdb.python_layerfx_drop_shadow(img, shoulderLayer, 
									      (87, 96, 0),   #color
												   75,   #opacity
													0,   #contou
													0,   #noise 
												   21,   #mode - grain merge 
												   20,   #spread 
												   40,   #size 
												  110,   #offset_angle 
												   10,   #offset_distance 
												 TRUE,   #knockout 
												FALSE)   #merge
	
	#Bevel and Emboss
	pdb.python_layerfx_bevel_emboss(img, shoulderLayer, 
													 3,   #style inner bevel
													65,   #depth 
													 1,   #direction 
													15,   #size 
													16,   #soften
												    90,   #angle 
													10,   #altitude 
													 7,   #gloss_contour - ring
									    (255, 144, 44),   #highlight_color 
													 4,   #highlight_mode - screen
												   100,   #highlight_opacity
											 ("black"),   #shadow_color 
													 3,   #shadow_mode - multiply
													75,   #shadow_opacity 
													 6,   #surface_contour - half round
												 FALSE,   #use_texture 
										   "Dried mud",   #pattern 
												   100,   #scale 
												   100,   #tex_depth 
												  TRUE,   #invert 
												 FALSE)   #merge	
	
	#Stem base
	baseLayer = pdb.gimp_layer_new(img, width, height, RGB_IMAGE, "Stem base", 50, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, baseLayer, 0)
	pdb.gimp_layer_add_alpha(baseLayer)
	pdb.gimp_drawable_fill(baseLayer, FILL_TRANSPARENT)
	pdb.gimp_brushes_set_brush("Stem base")
	pdb.gimp_context_set_brush_size(100)
	pdb.gimp_context_set_brush_default_hardness()
	pdb.gimp_context_set_brush_aspect_ratio(14)
	pdb.gimp_context_set_brush_spacing(0.01)
	pdb.gimp_context_set_brush_force(1.00)
	pdb.gimp_paintbrush_default(baseLayer, 2, [960, 230])
	
	#Bevel and Emboss
	pdb.python_layerfx_bevel_emboss(img, baseLayer, 
												 3,   #style inner bevel
												65,   #depth 
												 1,   #direction 
												 6,   #size 
												 5,   #soften
												90,   #angle 
												20,   #altitude 
												 0,   #gloss_contour - linear
								   (255, 255, 255),   #highlight_color 
											    21,   #highlight_mode - grain merge
											    30,   #highlight_opacity
										   (0,0,0),   #shadow_color 
												 3,   #shadow_mode - multiply
												75,   #shadow_opacity 
												 5,   #surface_contour - gaussian
											 FALSE,   #use_texture 
									   "Dried mud",   #pattern 
											   100,   #scale 
											   100,   #tex_depth 
											  TRUE,   #invert 
											 FALSE)   #merge
		
	#Stem
	stemLayer = pdb.gimp_layer_new(img, width, height, RGB_IMAGE, "Stem", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, stemLayer, 0)
	pdb.gimp_layer_add_alpha(stemLayer)
	pdb.gimp_drawable_fill(stemLayer, FILL_TRANSPARENT)
	pdb.gimp_brushes_set_brush("Orange stem")
	pdb.gimp_context_set_brush_size(140)
	pdb.gimp_context_set_brush_default_hardness()
	pdb.gimp_context_set_brush_aspect_ratio(0)
	pdb.gimp_context_set_brush_spacing(0.01)
	pdb.gimp_context_set_brush_force(1.00)
	pdb.gimp_paintbrush_default(stemLayer, 2, [968, 170])
	
	#Stem shadow
	pdb.script_fu_perspective_shadow(img, stemLayer, 
	                                            150,  #angle
												5.0,  #relative dist from horizon
												0.6,  #relative length fron shadow
												 14,  #blur radius
										    "black",  #color
												 80,  #opacity
												  1,  #interpolation
											  FALSE)  #allow resizing
	
	#Orange's Leaf
	leafLayer = pdb.gimp_layer_new(img, width, height, RGB_IMAGE, "Orange's leaf", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, leafLayer, 0)
	pdb.gimp_layer_add_alpha(leafLayer)
	pdb.gimp_drawable_fill(leafLayer, FILL_TRANSPARENT)
	pdb.gimp_brushes_set_brush("Orange's Leaf")
	pdb.gimp_context_set_brush_size(750)
	pdb.gimp_context_set_brush_default_hardness()
	pdb.gimp_context_set_brush_aspect_ratio(0)
	pdb.gimp_context_set_brush_spacing(0.54)
	pdb.gimp_context_set_brush_force(1.00)
	pdb.gimp_paintbrush_default(leafLayer, 2, [1200, 500])
		
	#Leaf shadow
	pdb.script_fu_perspective_shadow(img, leafLayer, 
	                                             94,   #angle
											    5.0,  #relative dist from horizon
											    0.9,  #relative length fron shadow
												 14,  #blur radius
											"black",  #color
												 80,  #opacity
												  1,  #interpolation
											  FALSE)  #allow resizing
	
	#Set gimp to default
	pdb.gimp_context_set_defaults()
	pdb.gimp_context_set_default_colors()
	
register(
	"pm_create_an_orange",
	"Creates an Orange from Conbagui's tutorial",
	"Creates an Orange from Conbagui's tutorial",
	"Pocholo",
	"Pocholo",
	"2020",
	"Create an Orange",
	"",
[
	(PF_BOOL, "tingeColor", "Add a Tinge Color?", TRUE),
],
[],
pm_create_an_orange, menu="<Image>/Pocholo-scripts/Create an Orange")
main()																											